@@ -375,13 +375,14 @@ def timepoint_consonance (data, method = 'cons', limit = 0.2, min_notes = 3):
     """
     
     data = np.moveaxis(data, 0, 1)
-    print('NAN', np.argwhere(np.isnan(data)))
+    #print('NAN', np.argwhere(np.isnan(data)))
     out = []
     positions = []
     for count, peaks in enumerate(data):
         peaks = [x for x in peaks if x >= 0]
         if method == 'cons':
             cons, b, peaks_cons, d = consonance_peaks(peaks, limit)
+            #print(peaks_cons)
             out.append(peaks_cons)
             if len(list(set(peaks_cons))) >= min_notes:
                 positions.append(count)
@@ -562,7 +563,6 @@ def scale_cons_matrix (scale, function):
         for index2 in range(len(scale)):
             if scale[index1] > scale[index2]:  #not include the diagonale in the computation of the avg. consonance
                 entry = scale[index1]/scale[index2]
-                #print(entry_value, scale[index1], scale[index2])
                 mode_values.append([scale[index1], scale[index2]])
                 metric_values.append(function(entry))
     return np.average(metric_values)
@@ -613,6 +613,26 @@ def scale_to_metrics(scale):
     return scale_metrics, scale_metrics_list
 
 
+def scale_consonance (scale, function, rounding = 4):
+    '''
+    Function that gives the average consonance of each scale interval
+    scale: List (float)
+        scale to reduce
+    function: function
+        function used to compute the consonance between pairs of ratios
+        Choose between: consonance, dyad_similarity, metric_denom
+    '''
+    metric_values = []
+    mode_values = []
+    for index1 in range(len(scale)):
+        metric_value = []
+        for index2 in range(len(scale)):          
+            entry = scale[index1]/scale[index2]
+            mode_values.append([scale[index1], scale[index2]])
+            metric_value.append(function(entry))
+        metric_values.append(np.average(metric_value))    
+    return metric_values
+
 '''
     ################################################   SCALE CONSTRUCTION  ##############################################################
 
@@ -979,7 +999,7 @@ def diss_curve (freqs, amps, denom=1000, max_ratio=2, euler_comp = True, method
     a = 1
     ratios_euler = [a]+ratios    
     ratios_euler = [int(round(num, 2)*1000) for num in ratios]
-    print(ratios_euler)
+    #print(ratios_euler)
     euler_score = None
     if euler_comp == True:
         euler_score = euler(*ratios_euler)
@@ -1311,29 +1331,31 @@ def harmonic_peaks_fit (peaks, amps, min_freq = 0.5, max_freq = 30, min_harms =
                     ratio = p/p2    
                     harm = -ratio
                 if ratio.is_integer():
-                    n += 1
-
-                    harm_temp.append(harm)
-
-                    harm_peaks_temp.append(p)
-                    harm_peaks_temp.append(p2)
+                    if harm <= harm_limit:
+                        n += 1
+                        harm_temp.append(harm)
+                        if p not in harm_peaks_temp:
+                            harm_peaks_temp.append(p)
+                        if p2 not in harm_peaks_temp:
+                                harm_peaks_temp.append(p2)
         n_total.append(n)
         harm_.append(harm_temp)
-        harm_peaks.append(list(set(harm_peaks_temp)))
+        harm_peaks.append(harm_peaks_temp)
         if n >= min_harms:
             max_n.append(n)
             max_peaks.append(p)
             max_amps.append(a)
-            #print(harm_peaks)
+            #print(harm_temp)
             harmonics.append(harm_temp)
             harmonic_peaks.append(harm_peaks)
-            harm_peaks_fit.append([p, harm_temp, list(set(harm_peaks_temp))])
+            harm_peaks_fit.append([p, harm_temp, harm_peaks_temp])
+    for i in range(len(harm_peaks_fit)):
+        harm_peaks_fit[i][2] = sorted(harm_peaks_fit[i][2])
     max_n = np.array(max_n)
     max_peaks = np.array(max_peaks)
     max_amps = np.array(max_amps)
     harmonics = np.array(harmonics)
-    for l in range(len(harmonics)):
-        harmonics[l] = [x for x in harmonics[l] if x < harm_limit]
+    #print(harmonics.shape)
     harmonic_peaks = np.array(harmonic_peaks)
     #harm_peaks_fit = np.array(harm_peaks_fit)
 
@@ -1342,7 +1364,7 @@ def harmonic_peaks_fit (peaks, amps, min_freq = 0.5, max_freq = 30, min_harms =
     return max_n, max_peaks, max_amps, harmonics, harmonic_peaks, harm_peaks_fit
 
 
-def cepstrum(signal, sample_freq, plot_cepstrum = False):
+def cepstrum(signal, sample_freq, plot_cepstrum = False, min_freq=1.5, max_freq=80):
     windowed_signal = signal
     dt = 1/sample_freq
     freq_vector = np.fft.rfftfreq(len(windowed_signal), d=dt)
@@ -1360,12 +1382,12 @@ def cepstrum(signal, sample_freq, plot_cepstrum = False):
         ax.plot(freq_vector, log_X)
         ax.set_xlabel('frequency (Hz)')
         ax.set_title('Fourier spectrum')
-        ax.set_xlim(0, 50)
+        ax.set_xlim(0, max_freq)
         fig, ax = plt.subplots()
         ax.plot(quefrency_vector, np.abs(cepstrum))
         ax.set_xlabel('quefrency (s)')
         ax.set_title('cepstrum')
-        ax.set_xlim(0.02, 0.5)
+        ax.set_xlim(1/max_freq, 1/min_freq)
         ax.set_ylim(0, 200)
     return cepstrum, quefrency_vector
 
@@ -1595,3 +1617,8 @@ def euclid_string_to_referent (strings, dict_rhythms):
         else:
             referent.append('None')
     return referent
+
+def euclid_long_to_short(pattern):
+    steps = len(pattern)
+    hits = pattern.count(1)
+    return [hits, steps]
\ No newline at end of file
