@@ -1,6 +1,7 @@
 import biotuner
 from biotuner.biotuner_functions import *
 from biotuner.biotuner_utils import *
+#from biotuner_utils import *
 from fooof import FOOOF
 from fooof.sim.gen import gen_power_spectrum
 from fooof.sim.utils import set_random_seed
@@ -106,8 +107,8 @@ class biotuner(object):
        Generates self.peaks and self.peaks_ratios attributes'''
 
     def peaks_extraction (self, data, peaks_function = None, FREQ_BANDS = None, precision = None, sf = None, min_freq = 1, max_freq = 60,
-                          min_harms = 2, compute_sub_ratios = None, ratios_extension = False, ratios_n_harms = None, scale_cons_limit =
-                          None, octave = 2, harm_limit = 128):
+                          min_harms = 2, compute_sub_ratios = False, ratios_extension = False, ratios_n_harms = None, scale_cons_limit =
+                          None, octave = 2, harm_limit=128, n_peaks=5, nIMFs=5, graph=False):
         '''
         
         Arguments
@@ -159,8 +160,20 @@ class biotuner(object):
             value of the octave
             
         harm_limit: int
-            Default to 128
+            Defaults to 128
             maximum harmonic position to keep when the 'harmonic_peaks' method is used
+        
+        n_peaks: int
+            Defaults to 5
+            number of peaks when using 'FOOOF' and 'cepstrum' peaks function. Peaks are chosen based on their amplitude.
+        
+        nIMFs: int
+            Defaults to 5
+            number of intrinsic mode functions to keep when using 'EEMD' or 'EMD' peaks function
+        
+        graph: boolean
+            Defaults to False
+            when set to True, a graph will accompanies the peak extraction method (except for 'fixed' and 'adapt')
             
         Attributes
         -------------
@@ -196,12 +209,16 @@ class biotuner(object):
             scale_cons_limit = self.scale_cons_limit
         if ratios_n_harms == None:
             ratios_n_harms = self.ratios_n_harms
-        self.peaks, self.amps = self.compute_peaks_ts (data, peaks_function = peaks_function, FREQ_BANDS = FREQ_BANDS, precision = precision, 
-                                                       sf = sf, min_freq = min_freq, max_freq = max_freq, min_harms = min_harms, harm_limit = harm_limit)
-
-
-        self.peaks_ratios = compute_peak_ratios(self.peaks, rebound = True, octave = octave, sub = compute_sub_ratios)
+        self.octave = octave
+        self.nIMFs = nIMFs
+        self.compute_sub_ratios = compute_sub_ratios
+        self.peaks, self.amps = self.compute_peaks_ts(data, peaks_function = peaks_function, FREQ_BANDS = FREQ_BANDS, precision =
+                                                       precision, sf = sf, min_freq = min_freq, max_freq = max_freq, min_harms = min_harms,
+                                                       harm_limit = harm_limit, n_peaks = n_peaks, graph=graph)
         
+        #print(self.peaks)
+        self.peaks_ratios = compute_peak_ratios(self.peaks, rebound = True, octave = octave, sub = compute_sub_ratios)
+        #print('peaks_ratios', self.peaks_ratios)
         self.peaks_ratios_cons, b = consonant_ratios (self.peaks, limit = scale_cons_limit)
         if ratios_extension == True:
             a, b, c = self.ratios_extension(self.peaks_ratios, ratios_n_harms = ratios_n_harms)
@@ -279,9 +296,9 @@ class biotuner(object):
     def compute_spectromorph (self, IMFs = None, sf = None, method = 'SpectralCentroid', window = None, overlap = 1, comp_chords = False, min_notes = 3, 
                               cons_limit = 0.2, cons_chord_method = 'cons', graph = False):
         if IMFs == None:
-            try:
+            if self.peaks_function == 'EEMD' or self.peaks_function == 'EMD':
                 IMFs = self.IMFs
-            except:
+            else:
                 IMFs = EMD_eeg(self.data)[1:6]
                 self.IMFs = IMFs
                 
@@ -290,42 +307,46 @@ class biotuner(object):
         if window == None:
             window = int(sf/2)
             
-        spectro_EMD = EMD_to_spectromorph(IMFs, 1000, method = method, window = window, overlap = overlap)
-        self.spectro_EMD = np.round(spectro_EMD, 1)
+        self.spectro_EMD = EMD_to_spectromorph(IMFs, sf, method = method, window = window, overlap = overlap)
         if method == 'SpectralCentroid':
             self.SpectralCentroid = self.spectro_EMD
         if method == 'SpectralFlux':
             self.SpectralFlux = self.spectro_EMD
         if comp_chords == True:
-            self.spectro_chords, spectro_chord_pos = timepoint_consonance(self.spectro_EMD, method = cons_chord_method, 
+            self.spectro_chords, spectro_chord_pos = timepoint_consonance(np.round(self.spectro_EMD, 1), method = cons_chord_method, 
                                                                            limit = cons_limit, min_notes = min_notes) 
             self.spectro_chords = [l[::-1] for l in self.spectro_chords]
         if graph == True:
             data = np.moveaxis(self.spectro_EMD, 0, 1)
             ax = sbn.lineplot(data=data[10:-10, :], dashes = False)
-            print('2')
+            #print('2')
             ax.set(xlabel='Time Windows', ylabel=method)
             ax.set_yscale('log')
             plt.legend(scatterpoints=1, frameon=True, labelspacing=1, title='EMDs', loc = 'best')
             labels = ['EMD1', 'EMD2', 'EMD3', 'EMD4', 'EMD5', 'EMD6']
-            for xc in spectro_chord_pos:
-                plt.axvline(x=xc, c='black', linestyle = 'dotted')
+            if comp_chords == True:
+                for xc in spectro_chord_pos:
+                    plt.axvline(x=xc, c='black', linestyle = 'dotted')
             plt.show()
             
     def compute_peaks_metrics (self, n_harm = None, bounds = 1, harm_bounds = 1):
         if n_harm == None:
             n_harm = self.n_harm
             
-        peaks = list(self.peaks)        
+        peaks = list(self.peaks)
+        peaks_ratios = compute_peak_ratios(peaks, rebound = True, octave = self.octave, sub = self.compute_sub_ratios)
         metrics = {'cons' : 0, 'euler' : 0, 'tenney': 0, 'harm_fit': 0, 'harmsim':0}   
-        harm_fit, _, _, harm_pos, common_harm_pos = harmonic_fit(peaks, n_harm = n_harm, bounds = harm_bounds)
-        metrics['harm_pos'] = harm_pos
-        metrics['common_harm_pos'] = common_harm_pos
-        metrics['harm_fit'] = len(harm_fit)
+        try:
+            harm_fit, _, _, harm_pos, common_harm_pos = harmonic_fit(peaks, n_harm = n_harm, bounds = harm_bounds)
+            metrics['harm_pos'] = harm_pos
+            metrics['common_harm_pos'] = common_harm_pos
+            metrics['harm_fit'] = len(harm_fit)
+        except:
+            pass
         a, b, c, metrics['cons'] = consonance_peaks (peaks, 0.1)
         peaks_euler = [int(round(num, 2)*1000) for num in peaks]
         
-        if self.peaks_function != 'cepstrum' and self.peaks_function != 'FOOOF' and self.peaks_function != 'harmonic_peaks':
+        if self.peaks_function == 'fixed' or self.peaks_function == 'adapt' or self.peaks_function == 'EMD' or self.peaks_function == 'EEMD':
             try:
 
                 metrics['euler'] = euler(*peaks_euler)
@@ -333,7 +354,7 @@ class biotuner(object):
             except:
                 pass
         metrics['tenney'] = tenneyHeight(peaks)
-        metrics['harmsim'] = np.average(ratios2harmsim(self.peaks_ratios[:-1]))
+        metrics['harmsim'] = np.average(ratios2harmsim(peaks_ratios))
         if self.peaks_function == 'harmonic_peaks':
             metrics['n_harmonic_peaks'] = self.n_harmonic_peaks
         metrics_list = []
@@ -362,7 +383,7 @@ class biotuner(object):
                                                                                   max_ratio=max_ratio, euler_comp = euler_comp,
                                                                                   method = method, plot = plot, n_tet_grid = n_tet_grid)
         self.diss_scale_cons, b = consonant_ratios (self.diss_scale, scale_cons_limit, sub = False, input_type = 'ratios')      
-        print('intervals', intervals)
+        #print('intervals', intervals)
         self.scale_metrics['diss_euler'] = euler_diss
         self.scale_metrics['dissonance'] = diss
         self.scale_metrics['diss_harm_sim'] = np.average(harm_sim_diss)
@@ -503,27 +524,69 @@ class biotuner(object):
         peaks = [p for p in peaks if p<=max_freq]
         return peaks, amps
 
-    def compute_peaks_ts (self, data, peaks_function = 'EMD', FREQ_BANDS = None, precision = 0.25, sf = 1000, min_freq = 1, max_freq = 80, min_harms = 2, harm_limit = 128):
+    def compute_peaks_ts (self, data, peaks_function = 'EMD', FREQ_BANDS = None, precision = 0.25, sf = None, min_freq=1, max_freq=80, min_harms = 2, harm_limit = 128, n_peaks = 5, nIMFs=None, graph=False):
         alphaband = [[7, 12]]
+        if sf == None:
+            sf = self.sf
+        if nIMFs == None:
+            nIMFs = self.nIMFs
         try:
             if FREQ_BANDS == None:
                 FREQ_BANDS = [[2, 3.55], [3.55, 7.15], [7.15, 14.3], [14.3, 28.55], [28.55, 49.4]]
         except:
             pass
         if peaks_function == 'EEMD':
-            IMFs = EMD_eeg(data)[1:6]
-            self.IMFs = IMFs
+            IMFs = EMD_eeg(data)
+            self.IMFs = IMFs[1:nIMFs+1]
+            IMFs = IMFs[1:nIMFs+1]
+            if graph == True:
+                t = np.linspace(0, len(data), len(data))  
+                nIMFs = len(IMFs)
+                plt.figure(figsize=(12,9))
+                plt.subplot(nIMFs+1, 1, 1)
+                plt.plot(t, data, 'r')
+                for n in range(nIMFs):
+                    plt.subplot(nIMFs+1, 1, n+2)
+                    plt.plot(t, IMFs[n], 'darkcyan')
+                    plt.ylabel("eIMF %i" %(n+1))
+                    plt.locator_params(axis='y', nbins=5)
+
+                plt.xlabel("Time [samples]")
+                plt.tight_layout()
+                plt.savefig('eemd_example', dpi=120)
+                plt.show()
         if peaks_function == 'EMD':
             data = np.interp(data, (data.min(), data.max()), (0, +1))
             IMFs = emd.sift.sift(data)
             #IMFs = emd.sift.ensemble_sift(data)
-            IMFs = np.moveaxis(IMFs, 0, 1)[1:6]
-            self.IMFs = IMFs
+            IMFs = np.moveaxis(IMFs, 0, 1)
+            self.IMFs = IMFs[1:nIMFs+1]
+            IMFs = IMFs[1:nIMFs+1]
+            if graph == True:
+                t = np.linspace(0, len(data), len(data))
+                eIMFs = self.IMFs
+                nIMFs = len(self.IMFs)
+
+
+                plt.figure(figsize=(12,9))
+                plt.subplot(nIMFs+1, 1, 1)
+                plt.plot(t, data, 'r')
+
+                for n in range(nIMFs):
+                    plt.subplot(nIMFs+1, 1, n+2)
+                    plt.plot(t, eIMFs[n], 'darkcyan')
+                    plt.ylabel("eIMF %i" %(n+1))
+                    plt.locator_params(axis='y', nbins=1)
+
+                plt.xlabel("Time [samples]")
+                plt.tight_layout()
+                plt.savefig('eemd_example', dpi=120)
+                plt.show()
         try:
             peaks_temp = []
             amps_temp = []
             for imf in range(len(IMFs)):
-                p, a = self.compute_peak(IMFs[imf], sf = self.sf, precision = precision, average = 'median')
+                p, a = self.compute_peak(IMFs[imf], sf = sf, precision = precision, average = 'median')
                 peaks_temp.append(p)
 
                 amps_temp.append(a)
@@ -535,7 +598,7 @@ class biotuner(object):
         if peaks_function == 'HH1D_max':
             IMFs = EMD_eeg(data)
             IMFs = np.moveaxis(IMFs, 0, 1)
-            IP, IF, IA = emd.spectra.frequency_transform(IMFs[:, 1:6], sf, 'nht')
+            IP, IF, IA = emd.spectra.frequency_transform(IMFs[:, 1:nIMFs+1], sf, 'nht')
             precision_hh = precision*2
             low = min_freq
             high = max_freq
@@ -544,7 +607,7 @@ class biotuner(object):
             self.IF = np.moveaxis(IF, 0 ,1)
             # Compute the 1d Hilbert-Huang transform (power over carrier frequency)
             spec = emd.spectra.hilberthuang_1d(IF, IA, edges)
-            
+            #print('spec', spec.shape)
             spec = np.moveaxis(spec, 0, 1)
             peaks_temp = []
             amps_temp = []
@@ -554,38 +617,60 @@ class biotuner(object):
                 amps_temp.append(spec[e][max_power])
             peaks_temp = np.flip(peaks_temp)
             amps_temp = np.flip(amps_temp)
+            peaks_temp = [np.round(p, 2) for p in peaks_temp]
+            amps_temp = [np.round(a, 2) for a in amps_temp]
+            if graph == True:
+                plt.figure(figsize=(8, 4))
+                plt.plot(bins, spec)
+                plt.xlim(min_freq, max_freq)
+                plt.xscale('log')
+                plt.xlabel('Frequency (Hz)')
+                plt.title('IA-weighted\nHilbert-Huang Transform')
+                plt.legend(['IMF-1', 'IMF-2', 'IMF-3', 'IMF-4', 'IMF-5', 'IMF-6', 'IMF-7'])
         #if peaks_function == 'HH1D_weightAVG':
 
         if peaks_function == 'adapt':
-            p, a = self.compute_peaks_raw(data, alphaband, precision = precision, average = 'median')
+            p, a = self.compute_peaks_raw(data, alphaband, sf = sf, precision = precision, average = 'median')
             FREQ_BANDS = alpha2bands(p)
-            peaks_temp, amps_temp = self.compute_peaks_raw(data, FREQ_BANDS, precision = precision, average = 'median')
+            peaks_temp, amps_temp = self.compute_peaks_raw(data, FREQ_BANDS, sf = sf, precision = precision, average = 'median')
         if peaks_function == 'fixed':
-            peaks_temp, amps_temp = self.compute_peaks_raw(data, FREQ_BANDS, precision = precision, average = 'median')
+            peaks_temp, amps_temp = self.compute_peaks_raw(data, FREQ_BANDS, sf = sf, precision = precision, average = 'median')
         if peaks_function == 'harmonic_peaks':
             p, a = self.extract_all_peaks(data, sf, precision, max_freq = sf/2)
             max_n, peaks_temp, amps_temp, self.harmonics, harm_peaks, harm_peaks_fit = harmonic_peaks_fit (p, a, min_freq, max_freq, min_harms = min_harms, harm_limit = harm_limit)
-            list_harmonics = functools_reduce(self.harmonics)
+            #list_harmonics = functools_reduce(self.harmonics)
+            list_harmonics = np.concatenate(self.harmonics)
             list_harmonics = list(set(abs(np.array(list_harmonics))))
             list_harmonics = [h for h in list_harmonics if h <= harm_limit]
             list_harmonics = np.sort(list_harmonics)
             self.all_harmonics = list_harmonics
             self.n_harmonic_peaks = len(peaks_temp)
+            self.harm_peaks_fit = harm_peaks_fit 
         if peaks_function == 'cepstrum':
-            cepstrum_, quefrency_vector = cepstrum(self.data, self.sf)
-            peaks_temp, amps_temp = cepstral_peaks(cepstrum_, quefrency_vector, 0.4, 0.02)          
-            peaks_temp = list(np.flip(peaks_temp))
-            peaks_temp = [np.round(p, 2) for p in peaks_temp]
-            amps_temp = list(np.flip(amps_temp))
+            if graph == True:
+                plot_cepstrum = True
+            if graph == False:
+                plot_cepstrum = False
+            cepstrum_, quefrency_vector = cepstrum(self.data, self.sf, min_freq=min_freq, max_freq=max_freq, plot_cepstrum=plot_cepstrum)
+            max_time = 1/min_freq
+            min_time = 1/max_freq
+            peaks_temp_, amps_temp_ = cepstral_peaks(cepstrum_, quefrency_vector, 1/min_freq, 1/max_freq)          
+            peaks_temp_ = list(np.flip(peaks_temp_))
+            peaks_temp_ = [np.round(p, 2) for p in peaks_temp_]
+            amps_temp_ = list(np.flip(amps_temp_))
+            peaks_temp = [x for _, x in sorted(zip(amps_temp_, peaks_temp_))][::-1][0:n_peaks]
+            amps_temp = sorted(amps_temp_)[::-1][0:n_peaks]
         if peaks_function == 'FOOOF':
             nfft = sf/precision
             nperseg = sf/precision
             freqs1, psd = scipy.signal.welch(data, self.sf, nfft = nfft, nperseg = nperseg)
             self.freqs = freqs1
             self.psd = psd
-            fm = FOOOF(peak_width_limits=[precision*2, 3], max_n_peaks=10, min_peak_height=0.2)
-            freq_range = [(sf/len(data))*2, 45]
+            fm = FOOOF(peak_width_limits=[precision*2, 3], max_n_peaks=50, min_peak_height=0.2)
+            freq_range = [(sf/len(data))*2, max_freq]
             fm.fit(freqs1, psd, freq_range)
+            if graph == True:
+                fm.report(freqs1, psd, freq_range)
             peaks_temp = []
             amps_temp = []
             for p in range(len(fm.peak_params_)):
@@ -594,6 +679,9 @@ class biotuner(object):
                     amps_temp.append(fm.peak_params_[p][1])
                 except:
                     pass
+            #print(peaks_temp)
+            peaks_temp = [x for _, x in sorted(zip(amps_temp, peaks_temp))][::-1][0:n_peaks]
+            amps_temp = sorted(amps_temp)[::-1][0:n_peaks]
             peaks_temp = [np.round(p, 2) for p in peaks_temp]
         if peaks_function == 'FOOOF_EEMD':
             nfft = sf/precision
@@ -607,14 +695,23 @@ class biotuner(object):
                 self.freqs = freqs1
                 self.psd = psd
                 fm = FOOOF(peak_width_limits=[precision*2, 3], max_n_peaks=10, min_peak_height=0.2)
-                freq_range = [(sf/len(data))*2, 45]
+                freq_range = [(sf/len(data))*2, max_freq]
                 fm.fit(freqs1, psd, freq_range)
+                if graph == True:
+                    fm.report(freqs1, psd, freq_range)
+                peaks_temp_EMD = fm.peak_params_[:, 0]
+                amps_temp_EMD = fm.peak_params_[:, 1]
+                #print('peaks', peaks_temp_EMD, 'amps', amps_temp_EMD)
                 try:
-                    peaks_temp.append(fm.peak_params_[0][0])
-                    amps_temp.append(fm.peak_params_[0][1])
+                    peaks_temp.append([x for _, x in sorted(zip(amps_temp_EMD, peaks_temp_EMD))][::-1][0:1])
+                    amps_temp.append(sorted(amps_temp_EMD)[::-1][0:1])
                 except:
-                    pass
+                    print('No peaks detected')
+                
             peaks_temp = [np.round(p, 2) for p in peaks_temp]
+            peaks_temp = [item for sublist in peaks_temp for item in sublist]
+            amps_temp = [item for sublist in amps_temp for item in sublist]
+        peaks_temp = [0.1 if x==0 else x for x in peaks_temp]
         peaks = np.array(peaks_temp)
         peaks = np.around(peaks, 3)
         amps = np.array(amps_temp)
