@@ -16,6 +16,10 @@ import itertools
 import operator
 import sys
 import bottleneck as bn
+from scipy import signal
+import neurokit2 as nk
+import pandas as pd
+from scipy.stats import pearsonr
 sys.setrecursionlimit(120000)
 
 
@@ -110,6 +114,13 @@ def rebound(x, low=1, high=2, octave=2):
 def rebound_list(x_list, low=1, high=2, octave=2):
     return [rebound(x, low, high, octave) for x in x_list] 
 
+
+def sum_list(l):
+    sum = 0
+    for x in l:
+        sum += x
+    return sum
+
 def nth_root (num, root):
     '''
     This function computes the ratio associated with one step of a N-TET scale
@@ -295,7 +306,7 @@ try:
 except:
     pass
 #Surrogates.silence_level = 1
-def AAFT_surrogates(self, original_data):
+def AAFT_surrogates(original_data):
     """
     Return surrogates using the amplitude adjusted Fourier transform
     method.
@@ -308,7 +319,7 @@ def AAFT_surrogates(self, original_data):
     :return: The surrogate time series.
     """
     #  Create sorted Gaussian reference series
-    gaussian = random.randn(original_data.shape[0], original_data.shape[1])
+    gaussian = np.random.randn(original_data.shape[0], original_data.shape[1])
     gaussian.sort(axis=1)
 
     #  Rescale data to Gaussian distribution
@@ -320,7 +331,7 @@ def AAFT_surrogates(self, original_data):
 
     #  Phase randomize rescaled data
     phase_randomized_data = \
-        correlated_noise_surrogates(self, rescaled_data)
+        correlated_noise_surrogates(rescaled_data)
 
     #  Rescale back to amplitude distribution of original data
     sorted_original = original_data.copy()
@@ -333,7 +344,7 @@ def AAFT_surrogates(self, original_data):
 
     return rescaled_data
 
-def correlated_noise_surrogates(self, original_data):
+def correlated_noise_surrogates(original_data):
     """
     Return Fourier surrogates.
 
@@ -373,8 +384,8 @@ def correlated_noise_surrogates(self, original_data):
     :rtype: 2D array [index, time]
     :return: The surrogate time series.
     """
-    if self.silence_level <= 1:
-        print("Generating correlated noise surrogates...")
+    #if self.silence_level <= 1:
+    #    print("Generating correlated noise surrogates...")
 
     #  Calculate FFT of original_data time series
     #  The FFT of the original_data data has to be calculated only once,
@@ -383,8 +394,8 @@ def correlated_noise_surrogates(self, original_data):
     #    surrogates = self._original_data_fft
     #else:
     surrogates = np.fft.rfft(original_data, axis=1)
-    self._original_data_fft = surrogates
-    self._fft_cached = True
+    _original_data_fft = surrogates
+    _fft_cached = True
 
     #  Get shapes
     (N, n_time) = original_data.shape
@@ -392,7 +403,7 @@ def correlated_noise_surrogates(self, original_data):
 
     #  Generate random phases uniformly distributed in the
     #  interval [0, 2*Pi]
-    phases = random.uniform(low=0, high=2 * np.pi, size=(N, len_phase))
+    phases = np.random.uniform(low=0, high=2 * np.pi, size=(N, len_phase))
 
     #  Add random phases uniformly distributed in the interval [0, 2*Pi]
     surrogates *= np.exp(1j * phases)
@@ -402,6 +413,39 @@ def correlated_noise_surrogates(self, original_data):
     return np.ascontiguousarray(np.real(np.fft.irfft(surrogates, n=n_time,
                                                      axis=1)))
 
+'''from: https://github.com/narayanps/NolinearTimeSeriesAnalysis/blob/master/SurrogateModule.py'''
+
+def UnivariateSurrogatesTFT(data_f,MaxIter=1,fc=5):
+    import numpy
+    xs=data_f.copy()
+    xs.sort() #sorted amplitude stored
+    pwx=numpy.abs(numpy.fft.fft(data_f)) # amplitude of fourier transform of orig
+    phi = numpy.angle(numpy.fft.fft(data_f))
+    Len=phi.shape[0]
+    #data_f.shape=(-1,1)
+    xsur = numpy.random.permutation(data_f) #random permutation as starting point
+    #xsur.shape = (1,-1)
+    #print(data_f.shape[0])
+    Fc =  numpy.round(fc*data_f.shape[0])
+    #print(Fc)
+    for i in range(MaxIter):
+        phi_surr = numpy.angle(numpy.fft.fft(xsur))
+        #print(phi_surr.shape)
+        #print(phi.shape)
+        phi_surr[1:Fc] = phi[1:Fc]
+        phi_surr[Len-Fc+1:Len] = phi[Len-Fc+1:Len]
+        phi_surr[0] = 0.0
+        new_len = int(Len/2)
+
+        phi_surr[new_len] = 0.0 
+              
+        fftsurx = pwx*numpy.exp(1j*phi_surr)
+        xoutb = numpy.real(numpy.fft.ifft(fftsurx))
+        ranks = xoutb.argsort(axis=0)
+        xsur[ranks] = xs
+    return(xsur)
+
+
 ## This function takes instantaneous frequencies (IF) from Hilbert-Huang Transform (HH) as an array in the form of [time, freqs]
 ## and outputs the average euler consonance for the whole time series (euler_tot) and the number of moments frequencies had consonance
 ## below euler threshold (euler_tresh)
@@ -487,6 +531,20 @@ def peaks_to_amps (peaks, freqs, amps, sf):
         amps_out.append(amp)
     return amps_out
 
+
+def alpha2bands(a):
+    np.float(a)
+    center_freqs = [a/4, a/2, a, a*2, a*4]
+    g_up = 1.618
+    g_down = 0.618
+    FREQ_BANDS = []
+    for f in center_freqs:
+        down = np.round((f/2)*1.618, 1)
+        up = np.round((f*2)*0.618, 1)
+        band = [down, up]
+        FREQ_BANDS.append(band)
+    return FREQ_BANDS
+
 def chords_to_ratios(chords, harm_limit = 2, spread = True):
     chords_ratios = []
     chords_ratios_bounded = []
@@ -564,7 +622,6 @@ def bjorklund(steps, pulses):
 '''Spectromorphology functions'''
 
 import pyACA
-
 def computeFeatureCl_new(afAudioData, cFeatureName, f_s, window=4000, overlap=1):
 
     # compute feature
@@ -584,13 +641,15 @@ def EMD_to_spectromorph (IMFs,  sf, method = "SpectralCentroid", window = None,
         f, t = computeFeatureCl_new(e, method, sf, window, overlap)
         #[f,t] = pyACA.computePitch('TimeAcf', e, 1000, afWindow=None, iBlockLength=1000, iHopLength=200)
         #df[i] = np.squeeze(f)
-        if method == 'SpectralCentroid':
-            try:
-                spectro_IMF.append(f[0][in_cut:out_cut])
-            except:
-                spectro_IMF.append(f[in_cut:out_cut])
-        if method == 'SpectralFlux':
+        #if method == 'SpectralCentroid':
+        try:
+            spectro_IMF.append(f[0][in_cut:out_cut])
+        except:
             spectro_IMF.append(f[in_cut:out_cut])
+        #if method == 'SpectralFlux':
+        #    spectro_IMF.append(f[in_cut:out_cut])
+        #else:
+        #    spectro_IMF.append(f[in_cut:out_cut])
     spectro_IMF = np.array(spectro_IMF)
     return spectro_IMF
 
@@ -689,6 +748,22 @@ sample_rate = 44100
 pygame.init()
 pygame.mixer.init(frequency=44100, size=-16, channels=2, buffer=512)
    
+    
+def generate_signal(sf, time_end, freqs, amps, show=False, theta = 0, color = 'blue'):
+    time = np.arange(0,time_end, 1/sf)
+    sine_tot  = []
+    for i in range(len(freqs)):
+        sinewave = amps[i] * np.sin(2 * np.pi * freqs[i] * time + theta)
+        sine_tot.append(sinewave)
+    sine_tot = sum_list(sine_tot)
+    if show == True:
+        
+        ax = plt.gca()
+        ax.set_facecolor('xkcd:black')
+        plt.plot(time, sine_tot, color = color)
+    return sine_tot
+
+
 def sine_wave(hz, peak, n_samples=sample_rate):
     """Compute N samples of a sine wave with given frequency and peak amplitude.
        Defaults to one second.
@@ -827,3 +902,13 @@ def convergents (interval):
     value = log2(interval)
     convergents = list(contfrac.convergents(value))
     return convergents
+
+def calculate_pvalues(df):
+    df = df.dropna()._get_numeric_data()
+    dfcols = pd.DataFrame(columns=df.columns)
+    pvalues = dfcols.transpose().join(dfcols, how='outer')
+    #print('p_values', pvalues)
+    for r in df.columns:
+        for c in df.columns:
+            pvalues[r][c] = round(pearsonr(df[r], df[c])[1], 4)
+    return pvalues
